#if LOGCAT_SERVER

using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEditor;
using System.Net;
using System.Net.Sockets;
using UnityEditor.TreeViewExamples;
using System.Linq;

public class Driver
{
    public string ip;
    public bool state;
}

public class LogCommon
{
    //获取本地ip地址
    public static string GetLocalIp()
    {
        string localIP = "";
        IPAddress[] addressList = Dns.GetHostEntry(Dns.GetHostName()).AddressList;
        foreach (IPAddress ip in addressList)
        {
            if (ip.AddressFamily == AddressFamily.InterNetwork)
            {
                localIP = ip.ToString();
                break;
            }
        }
        return localIP;
    }

    public static LogData Listaning(string ip)
    {
        return new LogData
        {
             ip      = ip,
             msg     = "Logcat listaning" ,
             logType = LogType.Log
        };
    }

    public static bool IsConnect(string msg)
    { 
        return msg.Contains("Logcat connect");
    }

    public static bool IsDisConnect(string msg)
    {
        return msg.Contains("Logcat disconnect");
    }
}

public class LogcatServer
{
    float intervalTime;
    LogUdpSend udpSend;
    LogUdpReceiver udpReceiver;
    Queue<LogData> logQueues => udpReceiver.logQueues;
    Dictionary<string, LogTreeElement> drivers = new Dictionary<string, LogTreeElement>();


    public System.Action changed;

    public List<LogTreeElement> logTrees=> drivers.Values.ToList();

    public void AddElement(LogTreeElement element)
    {
        drivers.Add(element.ip,element);
    }

    public void ClearElement()
    {
        drivers.Clear();
    }

    /// <summary>
    /// This function is called when the object becomes enabled and active.
    /// </summary>
    public void Init()
    {
        udpSend     = new LogUdpSend(9698);
        udpReceiver = new LogUdpReceiver(9696);

        EditorApplication.update += Update;
    }



    /// <summary>
    /// Update is called every frame, if the MonoBehaviour is enabled.
    /// </summary>
    void Update()
    {
        if (udpReceiver.logQueues.Count > 0)
        {
            while (logQueues.Count > 0)
            {
                Log(logQueues.Dequeue());
            }
        }

        intervalTime += Time.deltaTime;
        if (intervalTime > 1f)
        {
            intervalTime = 0;
            udpSend.Send("Logcat listaning");
        }
    }

    void Log(LogData log)
    {
        if (drivers.ContainsKey(log.ip))
        {
            if (drivers[log.ip].state)
            {
                var msg = $"{log.ip}:{log.msg}\n{log.stackTrace}";
                switch (log.logType)
                {
                    case LogType.Error:
                    case LogType.Exception: Debug.LogError(msg); break;
                    case LogType.Warning: Debug.LogWarning(msg); break;
                    case LogType.Log:
                    default: Debug.Log(msg); break;
                }
            }


            if (LogCommon.IsDisConnect(log.msg))
            {
                if (drivers.ContainsKey(log.ip))
                {
                    drivers[log.ip].state = false;
                    drivers.Remove(log.ip);
                    changed?.Invoke();
                }
            }
        }
        else
        {
            if (LogCommon.IsConnect(log.msg))
            {
                drivers.Add(log.ip, new LogTreeElement(log.ip,0,drivers.Count) { state = true });
            }
            else
            {
                drivers.Add(log.ip, new LogTreeElement(log.ip, 0, drivers.Count) {state = false });
            }
            changed?.Invoke();
        }

    }


    public void Release()
    {
        udpReceiver.Release();
        EditorApplication.update -= Update;
    }
}
#endif
